@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py 2441 2007-06-01 19:31:12Z wwalker $"
-__version__   = "$Revision: 2441 $"
-__date__      = "$Date: 2007-06-01 15:31:12 -0400 (Fri, 01 Jun 2007) $"
+__id__        = "$Id: atspi.py 2471 2007-06-13 13:44:34Z shaeger $"
+__version__   = "$Revision: 2471 $"
+__date__      = "$Date: 2007-06-13 09:44:34 -0400 (Wed, 13 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -823,12 +823,15 @@ class Accessible:
                 self.accessible.unref()
             except:
                 pass
+            
             try:
                 Accessible.deleteAccessible(self._acc)
-                self.accessible = None
-                self._acc = None
             except:
                 pass
+            
+            self.accessible = None
+            self._acc = None
+            self.app = None
 
     def __get_name(self):
         """Returns the object's accessible name as a string.
@@ -1227,6 +1230,25 @@ class Accessible:
             self.selection = selection
 
         return selection
+        
+    def __get_document(self):
+        """Returns an object that implements the Accessibility_Document
+        interface for this object, or None if this object doesn't implement
+        the Accessibility_Document interface.
+        """
+        document = self.accessible.queryInterface(\
+            "IDL:Accessibility/Document:1.0")
+
+        if document:
+            try:
+                document = document._narrow(Accessibility.Document)
+            except:
+                document = None
+
+        if document and settings.cacheValues:
+            self.document = document
+
+        return document
 
     def __get_table(self):
         """Returns an object that implements the Accessibility_Table
@@ -1252,7 +1274,6 @@ class Accessible:
         interface for this object, or None if this object doesn't implement
         the Accessibility_Text interface.
         """
-
         text = self.accessible.queryInterface("IDL:Accessibility/Text:1.0")
 
         if text:
@@ -1351,6 +1372,8 @@ class Accessible:
             return self.__get_value()
         elif attr == "attributes":
             return self.__get_attributes()
+        elif attr == "document":
+            return self.__get_document()
         elif attr.startswith('__') and attr.endswith('__'):
             raise AttributeError, attr
         else:
