@@ -22,13 +22,14 @@ level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py 1808 2006-12-12 03:40:50Z lmonsanto $"
-__version__   = "$Revision: 1808 $"
-__date__      = "$Date: 2006-12-11 22:40:50 -0500 (Mon, 11 Dec 2006) $"
+__id__        = "$Id: debug.py 2160 2007-03-12 17:18:51Z richb $"
+__version__   = "$Revision: 2160 $"
+__date__      = "$Date: 2007-03-12 13:18:51 -0400 (Mon, 12 Mar 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
+import sys
 logging.basicConfig()
 
 log = logging.getLogger()
@@ -153,6 +154,14 @@ def println(level, text = ""):
     - text: the text to print (default is a blank line)
     """
 
+    # Check to see if debugging is not enabled for the calling module.
+    #
+    frame = sys._getframe(1)
+    name = frame.f_globals["__name__"]
+    module = sys.modules[name]
+    if module.__dict__.has_key("_enableDebug") and not module._enableDebug:
+        return
+
     if level >= debugLevel:
         if debugFile:
             debugFile.writelines([text,"\n"])
@@ -207,3 +216,37 @@ def printDetails(level, indent, accessible, includeApp=True):
 
     if level >= debugLevel and accessible:
         println(level, accessible.toString(indent, includeApp))
+
+# The following code has been borrowed from the following URL:
+# 
+# http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html
+#
+import linecache
+
+def traceit(frame, event, arg):
+    """Line tracing utility to output all lines as they are executed by
+    the interpreter.  This is to be used by sys.settrace and is for 
+    debugging purposes.
+   
+    Arguments:
+    - frame: is the current stack frame
+    - event: 'call', 'line', 'return', 'exception', 'c_call', 'c_return',
+             or 'c_exception'
+    - arg:   depends on the event type (see docs for sys.settrace)
+    """ 
+
+    if event == "line":
+        lineno = frame.f_lineno
+        filename = frame.f_globals["__file__"]
+        if (filename.endswith(".pyc") or
+            filename.endswith(".pyo")):
+            filename = filename[:-1]
+        name = frame.f_globals["__name__"]
+        if name == "gettext" \
+           or name == "locale" \
+           or name == "posixpath" \
+           or name == "UserDict":
+            return traceit
+        line = linecache.getline(filename, lineno)
+        println(LEVEL_ALL, "TRACE %s:%s: %s" % (name, lineno, line.rstrip()))
+    return traceit
