@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: StarOffice.py 2414 2007-05-21 20:09:14Z richb $"
-__version__   = "$Revision: 2414 $"
-__date__      = "$Date: 2007-05-21 16:09:14 -0400 (Mon, 21 May 2007) $"
+__id__        = "$Id: StarOffice.py 2469 2007-06-12 20:19:09Z richb $"
+__version__   = "$Revision: 2469 $"
+__date__      = "$Date: 2007-06-12 16:19:09 -0400 (Tue, 12 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -478,7 +478,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         # If this is a table with no children, then let the user know.
         #
         if not obj.childCount:
-            # Translators: this indicates that there are zero items in 
+            # Translators: this indicates that there are zero items in
             # a layered pane or table.
             #
             utterances.append(_("0 items"))
@@ -870,38 +870,6 @@ class Script(default.Script):
         vbox.set_border_width(12)
         gtk.Widget.show(vbox)
 
-        # Text entry and label for "Enabled Text Attributes:".
-        #
-        hbox = gtk.HBox(False, 0)
-        gtk.Widget.show(hbox)
-
-        # Translators: All text in StarOffice/OpenOffice has a set of
-        # attributes associated with it (such as font type, size, style...)
-        # This label denotes those attributes that you user would like
-        # spoken when you ask to hear the text attributes associated with
-        # the current text (typically when the Insert-f key combination is 
-        # pressed).
-        #
-        self.textAttributesLabel = gtk.Label(_("Enabled Text Attributes:"))
-        gtk.Widget.show(self.textAttributesLabel)
-        gtk.Box.pack_start(hbox, self.textAttributesLabel, False, False, 5)
-
-        self.enabledTextAttributes = gtk.Entry()
-        gtk.Widget.show(self.enabledTextAttributes)
-        self.enabledTextAttributes.set_text(orca.settings.enabledTextAttributes)
-        gtk.Box.pack_end(hbox, self.enabledTextAttributes, True, True, 5)
-
-        acc_targets = []
-        acc_src = self.textAttributesLabel.get_accessible()
-        relation_set = acc_src.ref_relation_set()
-        acc_targ = self.enabledTextAttributes.get_accessible()
-        acc_targets.append(acc_targ)
-        relation = atk.Relation(acc_targets, 1)
-        relation.set_property('relation-type', atk.RELATION_LABEL_FOR)
-        relation_set.add(relation)
-
-        gtk.Box.pack_start(vbox, hbox, False, False, 0)
-
         # Checkbox for "Speak spread sheet cell coordinates".
         #
         # Translators: If checked, then Orca will speak the coordinates
@@ -911,7 +879,7 @@ class Script(default.Script):
         label = _("Speak spread sheet cell coordinates")
         self.speakCellCoordinatesCheckButton = gtk.CheckButton(label)
         gtk.Widget.show(self.speakCellCoordinatesCheckButton)
-        gtk.Box.pack_start(vbox, self.speakCellCoordinatesCheckButton, 
+        gtk.Box.pack_start(vbox, self.speakCellCoordinatesCheckButton,
                            False, False, 0)
         gtk.ToggleButton.set_active(self.speakCellCoordinatesCheckButton,
                                     speakCellCoordinates)
@@ -928,17 +896,36 @@ class Script(default.Script):
 
         global speakCellCoordinates
 
-        settings.enabledTextAttributes = self.enabledTextAttributes.get_text()
         prefs.writelines("\n")
-        prefs.writelines("orca.settings.enabledTextAttributes = '%s'\n" %  \
-                         settings.enabledTextAttributes)
-
         speakCellCoordinates = \
                  self.speakCellCoordinatesCheckButton.get_active()
         prefs.writelines( \
                  "orca.scripts.StarOffice.speakCellCoordinates = %s\n" % \
                  speakCellCoordinates)
 
+    def getAppState(self):
+        """Returns an object that can be passed to setAppState.  This
+        object will be use by setAppState to restore any state information
+        that was being maintained by the script."""
+        return [default.Script.getAppState(self),
+                self.dynamicColumnHeaders,
+                self.dynamicRowHeaders]
+
+    def setAppState(self, appState):
+        """Sets the application state using the given appState object.
+
+        Arguments:
+        - appState: an object obtained from getAppState
+        """
+        try:
+            [defaultAppState,
+             self.dynamicColumnHeaders,
+             self.dynamicRowHeaders] = appState
+            default.Script.setAppState(self, defaultAppState)
+        except:
+            debug.printException(debug.LEVEL_WARNING)
+            pass
+
     def adjustForWriterTable(self, obj):
         """Check to see if we are in Writer, where the object with focus
         is a paragraph, and the parent is the table cell. If it is, then,
@@ -1048,7 +1035,7 @@ class Script(default.Script):
         return inputLine
 
     def getSpreadSheetRowRange(self, obj):
-        """If this is spread sheet cell, return the start and end indices 
+        """If this is spread sheet cell, return the start and end indices
         of the spread sheet cells for the table that obj is in. Otherwise
         return the complete range (0, parent.table.nColumns).
 
