@@ -19,10 +19,10 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 2364 2007-05-09 20:15:44Z richb $"
-__version__   = "$Revision: 2364 $"
-__date__      = "$Date: 2007-05-09 16:15:44 -0400 (Wed, 09 May 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: where_am_I.py 2475 2007-06-15 14:25:41Z shaeger $"
+__version__   = "$Revision: 2475 $"
+__date__      = "$Date: 2007-06-15 10:25:41 -0400 (Fri, 15 Jun 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import atspi
@@ -31,8 +31,10 @@ import orca_state
 import rolenames
 import settings
 import speech
+import urlparse, urllib2
 
 from orca_i18n import _ # for gettext support
+from orca_i18n import ngettext  # for ngettext support
 
 class WhereAmI:
 
@@ -128,6 +130,9 @@ class WhereAmI:
 
         elif role == rolenames.ROLE_ICON:
             self._speakIconPanel(obj, doubleClick)
+            
+        elif role == rolenames.ROLE_LINK:
+            self._speakLink(obj, doubleClick)
 
         else:
             self._speakGenericObject(obj, doubleClick)
@@ -603,14 +608,131 @@ class WhereAmI:
         # and the count of the total number of icons within an icon panel. 
         # An example of an icon panel is the Nautilus folder view.
         #
-        utterances.append(_("%d of %d items selected") % \
-                          (totalSelectedItems, childCount))
+        itemString = ngettext("%d of %d item selected",
+                              "%d of %d items selected",
+                              childCount) % \
+                              (totalSelectedItems, childCount)
+        utterances.append(itemString)
 
         if doubleClick:
             for i in range(0, len(selectedItems)):
                 utterances.append(self._getObjLabelAndName(selectedItems[i]))
 
         speech.speakUtterances(utterances)
+        
+    def _speakLink(self, obj, doubleClick):
+        """Speaks information about a link including protocol, domain 
+        comparisons and size of file if possible
+        
+        Arguments:
+        - obj: the icon object that currently has focus.
+        - doubleClick: was it a doubleclick event?
+        """
+        
+        # get the URI for the link of interest and parse it.
+        # parsed URI is returned as a tuple containing six components: 
+        # scheme://netloc/path;parameters?query#fragment.
+        link_uri = self._script.getURI(obj)
+        if link_uri:
+            link_uri_info = urlparse.urlparse(link_uri)
+        else:
+            # something is wrong, just return
+            return
+      
+        # Try to get the URI of the active document and parse it
+        doc_uri = self.__getDocumentFrameURI(obj)
+        if doc_uri:
+            doc_uri_info = urlparse.urlparse(doc_uri)
+        else:
+            doc_uri_info = None
+              
+        # initialize our three outputs.  Output may change below for some 
+        # protocols.
+        # Translators: this is the protocol of a link eg. http, mailto.
+        #
+        linkoutput = _('%s link') %link_uri_info[0]
+        domainoutput = ''
+        sizeoutput = ''
+      
+        # get size and other protocol specific information
+        if link_uri_info[0] == 'ftp' or \
+           link_uri_info[0] == 'ftps' or \
+           link_uri_info[0] == 'file':
+            # change link output message to include filename
+            filename = link_uri_info[2].split('/')
+            linkoutput = _('%s link to %s') %(link_uri_info[0], filename[-1])
+            sizestr = self.__extractSize(link_uri)
+            sizeoutput = self.__formatSizeOutput(sizestr)
+   
+        # determine location differences if doc uri info is available
+        if doc_uri_info:
+            if link_uri_info[1] == doc_uri_info[1]:
+                if link_uri_info[2] == doc_uri_info[2]:
+                    # Translators: this is the domain relationship of a given
+                    # link to the current page.  eg. same page, same site.
+                    domainoutput = _('same page')
+                else:
+                    domainoutput = _('same site')
+            else:
+                # check for different machine name on same site
+                linkdomain = link_uri_info[1].split('.')
+                docdomain = doc_uri_info[1].split('.')
+                if len(linkdomain) > 1 and docdomain > 1  \
+                    and linkdomain[-1] == docdomain[-1]  \
+                    and linkdomain[-2] == docdomain[-2]:
+                    domainoutput = _('same site') 
+                else:
+                    domainoutput = _('different site')
+
+        speech.speakUtterances([linkoutput, domainoutput, sizeoutput])
+      
+    def __extractSize(self, uri):
+        '''
+        Get the http header for a given uri and try to extract the size (Content-length).
+        '''
+        try:
+            x=urllib2.urlopen(uri)
+            try:
+                return x.info()['Content-length']
+            except KeyError:
+                return None
+        except (ValueError, urllib2.URLError, OSError):
+            return None
+  
+    def __formatSizeOutput(self, sizestr):
+        '''
+        Format the size output announcement.  Changes wording based on size.
+        '''
+        # sanity check
+        if sizestr is None or sizestr == '':
+            return ''
+        size = int(sizestr)
+        if size < 10000:
+            # Translators: This is the size of a file in bytes
+            #
+            return ngettext('%d byte', '%d bytes', size) % size
+        elif size < 1000000:
+            # Translators: This is the size of a file in kilobytes
+            #
+            return _('%.2f kilobytes') % (float(size) * .001)
+        elif size >= 1000000:
+            # Translators: This is the size of a file in megabytes
+            #
+            return _('%.2f megabytes') % (float(size) * .000001) 
+        
+    def __getDocumentFrameURI(self, obj):
+        '''
+        Returns the URI of the document frame that is active
+        '''
+        try:
+            documentFrame = self._script.getDocumentFrame()
+        except AttributeError:
+            return None
+        attrs = documentFrame.document.getAttributes()
+        for attr in attrs:
+            if attr.startswith('DocURL'):
+                return attr[7:]
+        return None
 
     def _speakGenericObject(self, obj, doubleClick):
         """Speak a generic object; one not specifically handled by
@@ -869,6 +991,165 @@ class WhereAmI:
 
         return text
 
+    def _getTextSelection(self, obj):
+        """Get the text selection for the given object.
+
+        Arguments:
+        - obj: the text object to extract the selected text from.
+
+        Returns: the selected text contents plus the start and end 
+        offsets within the text.
+        """
+
+        textContents = ""
+        textObj = obj.text
+        nSelections = textObj.getNSelections()
+        for i in range(0, nSelections):
+            [startOffset, endOffset] = textObj.getSelection(i)
+
+            debug.println(self._debugLevel,
+                "_getTextSelection: selection start=%d, end=%d" % \
+                (startOffset, endOffset))
+
+            selectedText = textObj.getText(startOffset, endOffset)
+            debug.println(self._debugLevel,
+                "_getTextSelection: selected text=<%s>" % selectedText)
+
+            if i > 0:
+                textContents += " "
+            textContents += selectedText
+
+        return [textContents, startOffset, endOffset]
+
+    def _getTextSelections(self, obj, doubleClick):
+        """Get all the text applicable text selections for the given object.
+        If the user doubleclicked, look to see if there are any previous 
+        or next text objects that also have selected text and add in their 
+        text contents.
+
+        Arguments:
+        - obj: the text object to start extracting the selected text from.
+        - doubleClick: True if the user double-clicked the "where am I" key.
+
+        Returns: all the selected text contents plus the start and end
+        offsets within the text for the given object.
+        """
+
+        textContents = ""
+        startOffset = 0
+        endOffset = 0
+        if obj.text.getNSelections() > 0:
+            [textContents, startOffset, endOffset] = \
+                                            self._getTextSelection(obj)
+
+        if doubleClick:
+            current = obj
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                for relation in current.relations:
+                    if relation.getRelationType() == \
+                           atspi.Accessibility.RELATION_FLOWS_FROM:
+                        prevObj = atspi.Accessible.makeAccessible( \
+                                                  relation.getTarget(0))
+                        if prevObj.text.getNSelections() > 0:
+                            [newTextContents, start, end] = \
+                                         self._getTextSelection(prevObj)
+                            textContents = newTextContents + textContents
+                            current = prevObj
+                            morePossibleSelections = True
+                        else:
+                            displayedText = prevObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current = prevObj
+                                morePossibleSelections = True
+                        break
+
+            current = obj
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                for relation in current.relations:
+                    if relation.getRelationType() == \
+                           atspi.Accessibility.RELATION_FLOWS_TO:
+                        nextObj = atspi.Accessible.makeAccessible( \
+                                                  relation.getTarget(0))
+                        if nextObj.text.getNSelections() > 0:
+                            [newTextContents, start, end] = \
+                                         self._getTextSelection(nextObj)
+                            textContents += newTextContents
+                            current = nextObj
+                            morePossibleSelections = True
+                        else:
+                            displayedText = nextObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current = nextObj
+                                morePossibleSelections = True
+                        break
+
+        return [textContents, startOffset, endOffset]
+
+    def _hasTextSelections(self, obj):
+        """Return an indication of whether this object has selected text.
+        Note that it's possible that this object has no text, but is part
+        of a selected text area. Because of this, we need to check the
+        objects on either side to see if they are none zero length and
+        have text selections.
+
+        Arguments:
+        - obj: the text object to start checking for selected text.
+
+        Returns: an indication of whether this object has selected text,
+        or adjacent text objects have selected text.
+        """
+
+        currentSelected = False
+        otherSelected = False
+        nSelections = obj.text.getNSelections()
+        if nSelections:
+            currentSelected = True
+        else:
+            otherSelected = False
+            displayedText = obj.text.getText(0, -1)
+            if len(displayedText) == 0:
+                current = obj
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    for relation in current.relations:
+                        if relation.getRelationType() == \
+                               atspi.Accessibility.RELATION_FLOWS_FROM:
+                            prevObj = atspi.Accessible.makeAccessible( \
+                                                      relation.getTarget(0))
+                            if prevObj.text.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = prevObj.text.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current = prevObj
+                                    morePossibleSelections = True
+                            break
+
+                current = obj
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    for relation in current.relations:
+                        if relation.getRelationType() == \
+                               atspi.Accessibility.RELATION_FLOWS_TO:
+                            nextObj = atspi.Accessible.makeAccessible( \
+                                                      relation.getTarget(0))
+                            if nextObj.text.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = nextObj.text.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current = nextObj
+                                    morePossibleSelections = True
+                            break
+
+        return [currentSelected, otherSelected]
+
     def _getTextContents(self, obj, doubleClick):
         """Returns utterences for text.
 
@@ -890,23 +1171,12 @@ class WhereAmI:
             "_getTextContents: caretOffset=%d, nSelections=%d" % \
             (caretOffset, nSelections))
 
-        if nSelections:
+        [current, other] = self._hasTextSelections(obj)
+        if (doubleClick and (current or other)) or \
+           (not doubleClick and current):
             selected = True
-            for i in range(0, nSelections):
-                [startOffset, endOffset] = textObj.getSelection(i)
-
-                debug.println(self._debugLevel,
-                    "_getTextContents: selection start=%d, end=%d" % \
-                    (startOffset, endOffset))
-
-                selectedText = textObj.getText(startOffset, endOffset)
-                debug.println(self._debugLevel,
-                    "_getTextContents: selected text=<%s>" % selectedText)
-
-                if i > 0:
-                    textContents += " "
-                textContents += selectedText
-
+            [textContents, startOffset, endOffset] = \
+                                  self._getTextSelections(obj, doubleClick)
         else:
             # Get the line containing the caret
             #
